<?php defined('SYSPATH') or die('No direct script access.');

class FileParser
{
    /**
     * Parse students' info & synchronize it with database.
     * @param $filename string File with source data
     * @param $facultyID int
     * @return array errors
     */
    public static function uploadStudents($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");

        $i = 0;
        $errors = [];

        while ($line = fgetcsv($file, 0, ";")) {
            // ФИО, курс, степень подготовки,
            list($name, $gradeNum, $groupNum, $degree, $spec) = UTF8::clear($line);

            // Фамилия, имя, отчество
            list($lastName, $firstName, $secondName) = self::parseFullName($name);

            $degree = self::getDegreeType($degree);

            $attempt = Model_Account::createStudentEx(
                $lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $spec, $facultyID
            );

            if ($attempt == -1) {
                $errors[] = ['Row' => $i, 'Info' => implode(';', $line)];
            }

            $i++;
        }

        return $errors;
    }

    private static function getDegreeType($degree) {
        switch ($degree) {
            case 'Бакалавр':
                return 'bachelor';
            case 'Магистр':
                return 'master';
            case 'Специалист':
                return 'specialist';
            default:
                return $degree;
        }
    }

    /**
     * Parse subjects' info & synchronize it with database.
     * @param $filename  string File with source data
     * @param $facultyID int
     * @return array
     */
    public static function uploadSubjects($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");
        $count = $exists = 0;
        $errors = [];

        while ($line = fgetcsv($file, 0, ";")) {
            list($name, $abbr) = UTF8::clear($line);
            $attempt = Model_Subject::create($name, $abbr, $facultyID);

            if ($attempt < 0) {
                $errors[] = ['Row' => $count, 'Info' => implode(';', $line)];
            } elseif ($attempt == 1) {
                $exists++;
            }

            $count++;
        }

        return [
            'Errors'             => $errors,
            'ErrorsCount'        => count($errors),
            'RecordsCount'       => $count,
            'RecordsExistsCount' => $exists,
        ];
    }


    /**
     * Parse teachers' info & synchronize it with database.
     * @param $filename  string File with source data
     * @param $facultyID int
     * @return array
     */
    public static function uploadTeachers($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

//        $faculties = [];
//        foreach (Model_Faculties::load() as $faculty) {
//            $name = trim($faculty['Name']);
//            $faculties[$name] = $faculty['ID'];
//        }

        $count = 0;
        $errors = [];
        $file = fopen($filename, "r");

        while ($line = fgetcsv($file, 0, ";")) {
            list($name, $department, $facultyName) = UTF8::clear($line);

            // Фамилия, имя, отчество
            list($lastName, $firstName, $secondName) = self::parseFullName($name);

//            if ($facultyName)
//                $facultyID = $faculties[trim($facultyName)];

            $attempt = Model_Account::createTeacherByDepName($lastName, $firstName, $secondName, $department, $facultyID);

            if ($attempt == -1) {
                $errors[] = ['Row' => $count, 'Info' => implode(';', $line)];
            }

            $count++;
        }

        return $errors;
    }

    private static function parseFullName($name) {
        // Енотова (Сенченко) Наталья Лин Чу Геннадьевна
        $name = preg_replace('/\(.*\)\s+/', '', $name);
        $res = explode(' ', $name, 3);
        return Arr::map('trim', $res);
    }
}